#include<iostream>
#include<string>
#include<string.h>
#include<stack>
#include<assert.h>
#include<stdexcept>

using namespace std;
using std::ostream;

template <class T>
//class SeqQueue : public Queue<T> {
class SeqQueue {
public:
    /** Initialize your data structure here. Set the size of the deque to be k. */
    SeqQueue(int k=10); 
    ~SeqQueue(); 

    /** Adds an item at the front of Deque. Return true if the operation is successful. */
    bool insertFront(T value);
    
    /** Adds an item at the rear of Deque. Return true if the operation is successful. */
    bool insertLast(T value);
    
    /** Deletes an item from the front of Deque. Return true if the operation is successful. */
    bool deleteFront();
    
    /** Deletes an item from the rear of Deque. Return true if the operation is successful. */
    bool deleteLast();
    
    /** Get the front item from the deque. */
    T getFront();
    
    /** Get the last item from the deque. */
    T getRear();
    
    /** Checks whether the circular deque is empty or not. */
    bool isEmpty();
    
    /** Checks whether the circular deque is full or not. */
    bool isFull();
    bool printQueue();
private:
    int capacity;
    int rear, front;
    T *elements;
};

/** Initialize your data structure here. Set the size of the deque to be k. */
template <typename T>
SeqQueue<T>::SeqQueue(int k) {
    front = 0;
    rear = 0;
    capacity = k;
    elements = new T[capacity];
    assert(elements != nullptr);
}

template <typename T>
SeqQueue<T>::~SeqQueue() {
    delete []elements;
}

/** Adds an item at the front of Deque. Return true if the operation is successful. */
template <typename T>
bool SeqQueue<T>::insertFront(T value) {
    if(isFull()) {
        return false; 
    }
    elements[front] = value;
    front = (front - 1 + capacity) % capacity;
    return true;
}

/** Adds an item at the rear of Deque. Return true if the operation is successful. */
template <typename T>
bool SeqQueue<T>::insertLast(T value) {
    if(isFull()) {
        return false; 
    }
    rear = (rear + 1) % capacity;
    elements[rear] = value;
    return true;
}

/** Deletes an item from the front of Deque. Return true if the operation is successful. */
template <typename T>
bool SeqQueue<T>::deleteFront() {
    if(isEmpty()) {
        return false;
    }
    front = (front + 1) % capacity;
    return true;
}

/** Deletes an item from the rear of Deque. Return true if the operation is successful. */
template <typename T>
bool SeqQueue<T>::deleteLast() {
    if(isEmpty()) {
        return false;
    }
    rear = (rear - 1 + capacity) % capacity;
    return true;
}

/** Get the front item from the deque. */
template <typename T>
T SeqQueue<T>::getFront() {
    if(isEmpty()) {
	std::logic_error e("The queue is empty");
        throw std::exception(e);
    }
    return elements[(front + 1) % capacity];
}

/** Get the last item from the deque. */
template <typename T>
T SeqQueue<T>::getRear() {
    if(isEmpty()) {
	std::logic_error e("The queue is empty");
        throw std::exception(e);
    }
    return elements[rear];
}

/** Checks whether the circular deque is empty or not. */
template <typename T>
bool SeqQueue<T>::isEmpty() {
    if(front == rear) {
        return true;
    }
    return false;
}

/** Checks whether the circular deque is full or not. */
template <typename T>
bool SeqQueue<T>::isFull() {
    if((rear + 1) % capacity == front % capacity) {
        return true;
    }
    return false;
}

/** traverse circular deque. */
template <typename T>
bool SeqQueue<T>::printQueue() {
    if(isEmpty()) {
	std::logic_error e("The queue is empty");
        throw std::exception(e);
    } else {
	int p = front;
        while(p != rear) {
	    p = (p + 1) % capacity;
	    cout << elements[p] << endl;    
	}
    }

    return true;
}

    
/**
 * Your SeqQueue object will be instantiated and called as such:
 * SeqQueue* obj = new SeqQueue(k);
 * bool param_1 = obj->insertFront(value);
 * bool param_2 = obj->insertLast(value);
 * bool param_3 = obj->deleteFront();
 * bool param_4 = obj->deleteLast();
 * int param_5 = obj->getFront();
 * int param_6 = obj->getRear();
 * bool param_7 = obj->isEmpty();
 * bool param_8 = obj->isFull();
 */

string stringToString(string input) {
    assert(input.length() >= 2);
    string result;
    for (int i = 1; i < input.length() -1; i++) {
        char currentChar = input[i];
        if (input[i] == '\\') {
            char nextChar = input[i+1];
            switch (nextChar) {
                case '\"': result.push_back('\"'); break;
                case '/' : result.push_back('/'); break;
                case '\\': result.push_back('\\'); break;
                case 'b' : result.push_back('\b'); break;
                case 'f' : result.push_back('\f'); break;
                case 'r' : result.push_back('\r'); break;
                case 'n' : result.push_back('\n'); break;
                case 't' : result.push_back('\t'); break;
                default: break;
            }
            i++;
        } else {
          result.push_back(currentChar);
        }
    }
    return result;
}

string boolToString(bool input) {
    return input ? "True" : "False";
}

int main() {
#if 0
    string line;
    while (getline(cin, line)) {
        string s = stringToString(line);
    }
#endif
    SeqQueue<int> sQueue(16);

    sQueue.insertLast(10);
    sQueue.insertLast(20);
    sQueue.insertLast(30);
    sQueue.insertLast(40);
    sQueue.insertLast(50);
    sQueue.insertLast(60);
    sQueue.insertLast(70);
    sQueue.insertLast(80);
    sQueue.insertLast(90);
    sQueue.insertLast(100);
    sQueue.insertLast(110);

    sQueue.printQueue();
    cout << "printQueue end" << endl;
    cout << "Queue front data = " << sQueue.getFront() << endl;
    cout << "Queue rear data = " << sQueue.getRear() << endl;

    sQueue.deleteFront();
    sQueue.deleteFront();
    sQueue.deleteLast();
    sQueue.deleteLast();
    sQueue.printQueue();
    cout << "printQueue end" << endl;
    cout << "Queue front data = " << sQueue.getFront() << endl;
    cout << "Queue rear data = " << sQueue.getRear() << endl;

    sQueue.insertFront(120);
    sQueue.insertFront(90);
    sQueue.insertLast(240);
    sQueue.insertLast(360);
    sQueue.insertLast(480);
    sQueue.insertLast(570);
    sQueue.printQueue();
    cout << "printQueue end" << endl;
    cout << "Queue front data = " << sQueue.getFront() << endl;
    cout << "Queue rear data = " << sQueue.getRear() << endl;

    return 0;
}
